Skip to main content

Pypi version Publish Package Licence PyPI Downloads Python versions

pywebexec

Simple Python HTTP(S) API/Web Server Command Launcher and Terminal sharing

  • build a Restfull API/swagger-ui powered application in no time exposing simple commands/parameters.
  • create a toolbox with batch management/parallel execution of commands
  • share a terminal in one command

Install

$ pip install pywebexec

Quick start

  • share terminal
    • start http server and spawn a new terminal shared on 0.0.0.0 port 8080 (defaults)
    • exiting terminal stops server/share
$ pywebexec shareterm
  • serve executables
    • put in a directory the scripts/commands/links to commands you want to expose
    • start http server serving current directory executables listening on 0.0.0.0 port 8080
$ pywebexec -d <dir>
  • Launch commands with params/view live output/Status using browser
  • Share your terminal output using pywebexec -d <dir> term

pywebexecnew11

all commands output / statuses are available in the executables directory in subdirectory .web_status

features

  • Serve executables in a directory
  • full API driven with dynamic swagger UI
  • Launch commands with params from web browser or API call
  • multiple share terminal output
  • Follow live output
  • Replay terminal history
  • Stop command
  • Relaunch command
  • HTTPS support
  • HTTPS self-signed certificate generator
  • Basic Auth
  • LDAP(S) password check/group member
  • Safe url token generation
  • Can be started as a daemon (POSIX)
  • Uses gunicorn to serve http/https
  • Linux/MacOS compatible
  • Markdown help for commands
  • YAML schema for commands parameters
  • Batch/parallel command execution

Customize server

$ pywebexec --dir ~/myscripts --listen 0.0.0.0 --port 8080 --title myscripts
$ pywebexec -d ~/myscripts -l 0.0.0.0 -p 8080 -t myscripts

Sharing terminals

  • start server and share tty in one command
$ pywebexec -d ~/webshare shareterm
  • share tty with an already pywebexec server started
$ pywebexec -d ~/webshare term

if another user need to share his terminal, he need to have write permission on <dir>/.web_status directory.

Safe url token

  • generate safe url, use the url to access the server
$ pywebexec -T
$ pywebexec --tokenurl
Starting server:
http://<host>:8080?token=jSTWiNgEVkddeEJ7I97x2ekOeaiXs2mErRSKNxm3DP0
http://x.x.x.x:8080?token=jSTWiNgEVkddeEJ7I97x2ekOeaiXs2mErRSKNxm3DP0

Basic auth

  • single user/password
$ pywebexec --user myuser [--password mypass]
$ pywebexec -u myuser [-P mypass]

Generated password is given if no --pasword option

  • ldap(s) password check / group member ldap server must accept memberOf attribute for group members
$ export PYWEBEXEC_LDAP_SERVER=ldaps://ldap.mydomain.com:389
$ export PYWEBEXEC_LDAP_BIND_DN="cn=read-only-admin,dc=example,dc=com"
$ export PYWEBEXEC_LDAP_BIND_PASSWORD="password"
$ export PYWEBEXEC_LDAP_BASE_DN="dc=example,dc=com"
$ export PYWEBEXEC_LDAP_USER_ID="uid" # sAMAccountName for AD
$ export PYWEBEXEC_LDAP_GROUPS="ou=mathematicians,dc=example,dc=com ou=scientists,dc=example,dc=com"
$ pywebexec

HTTPS server

  • Generate auto-signed certificate and start https server
$ pywebexec --gencert
$ pywebexec --g
  • Start https server using existing certificate
$ pywebexec --cert /pathto/host.cert --key /pathto/host.key
$ pywebexec -c /pathto/host.cert -k /pathto/host.key

Launch server as a daemon

$ pywebexec start
$ pywebexec status
$ pywebexec stop
  • log of server are stored in directory ~/[.config/].pywebexec/pywebexec_<listen>:<port>.log

Launch command through API

$ curl http://myhost:8080/commands/myscript -H 'Content-Type: application/json' -X POST -d '{"params":["param1", ...]}'
$ curl http://myhost:8080/commands/<command_id>
$ curl http://myhost:8080/commands/<command_id>/output -H "Accept: text/plain"

Add markdown help to commands

For each exposed command, you can add a help message by creating a file named <command>.help in the same directory as the command. The help message must be written in markdown.
The help message is displayed:

  • in the web interface as tooltip when focused on param input field,
  • in the response when calling the API /executables
  • in the swagger-ui in the /commands/<command> route.

Add schema to commands

For each exposed command, you can add a schema by creating a file named <command>.schema.yaml in the same directory as the command. The schema must be written in yaml format.
The schema is used to generate a form in the web interface and in the swagger-ui in the /commands/<command> route.
The schema is also used to validate the input parameters when calling the API /commands/<command>.
The schema must be written in the openapi schema format.

type: object
properties:
  param1:
    type: string
    description: "param1 description"
    example: "value"
  param2:
    type: integer
    description: "param2 description"
    enum: [1, 2, 3]
  param3:
    type: array
    items:
      type: string
    description: "param3 description"
    example: ["value1", "value2"]
required:
  - param1
  - param2

The payload will be converted to command line arguments when calling the command.

command --param1 value --param2 1 --param3 value1 value2
  • On the web inferface, and swagger-ui the form will be generated from the schema.

When using schema, the command can now be launched with:

$ curl -X POST http://<srv>/commands/<cmd> -H "Content-Type: application/json" -d '{"param1": "value", "param2": 1, "param3": ["value1", "value2"]}'

Schema options

The schema options are used to customize the command line arguments generation, just add a schema_options section to the schema.

schema_options:
  separator_params: {"*": " ", "param2": "="}}"=" # --param2=value (default is " ") 
  noprefix_params: ["param1", "param2"] # omit --param prefix, use "*" to omit all
  convert_params: {"param1": "param2"} # convert param1 to param2

Batch commands/parallel execution

Integration of run-para to enable batch execution of commands:

  • In schema_options adding batch_param will enable batch mode for the command, the command will be executed for each value in the batch_param list.
  • The batch_param is the name of the parameter that will be used to pass the different values for the parameter.
  • The batch_param type will be transformed to textarea to provide list to use as parameter for the command.
  • The range parameters parallel and delay is added to the command parameters to control the execution of the batch commands (nb jobs in parallel and initial delay between jobs).

Swagger UI

A custom swagger UI is available at http[s]://<srv>/v0/documentation with enhanced markdown rendering and form generation for body parameters.

API reference

method route params/payload returns
GET /commands/exposed commands: [
  {
    command: str,
    help: str
  },
]
GET /commands commands: [
  {
    command_id: uuid
    command: str
    start_time: isotime
    end_time: isotime
    status: str
    exit_code: int
    last_output_line: str
  },
]
GET /commands/{id} command_id: uuid
command: str
params: array[str]
start_time: isotime
end_time: isotime
status: str
exit_code: int
last_output_line: str
GET /commands/{id}/output offset: int output: str
status: str
links: { next: str }
GET /commands/{id}/output_raw offset: int output: stream raw output until end of command
curl -Ns http://srv/commands/{id}/output_raw
POST /commands command: str
params: array[str]
rows: int
cols: int
command_id: uuid
message: str
POST /commands/{cmd} params: array[str]
rows: int
cols: int
command_id: uuid
message: str
PATCH /commands/{id}/stop message: str
  • to get command output as text (without ANSI codes/Control characters) use: /commands/{id}/output with header "Accept: text/plain"

Metadata

Release files for pywebexec 2.4.28

For a detailed explanation of source distributions (sdists) and built distributions (wheels), please see the package formats documentation.

Source distribution (sdist)

Source distribution for pywebexec 2.4.28
File Size Uploaded
pywebexec-2.4.28.tar.gz 3.1 MB Details

Built distribution (wheel)

Table of built distributions (wheels) for pywebexec 2.4.28
File Interpreter ABI Platform
pywebexec-2.4.28-py3-none-any.whl Python 3 none any Details

Total release size: 6.3 MB

Release files / pywebexec-2.4.28.tar.gz

Download URL pywebexec-2.4.28.tar.gz
Size 3.1 MB
Tags Source
SHA-256 checksum
How to use checksums
6bb9f539b022728e4579bb29fd515d0bfb3f38dbba612e3b4f70865cce11207a
BLAKE2b-256 checksum
How to use checksums
b7a42eb8b4e59aec743d4b7811cd066bbf07a55cd6ec290e2360fb1718adf75e
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
No
Uploaded via twine/6.1.0 CPython/3.9.22

Release files / pywebexec-2.4.28-py3-none-any.whl

Download URL pywebexec-2.4.28-py3-none-any.whl
Size 3.1 MB
Tags Python 3
SHA-256 checksum
How to use checksums
7eb7e67d5d07962811ce3b5313054ef47704c279ea2770799c78319cf628846e
BLAKE2b-256 checksum
How to use checksums
fc7a5619d22ee4be6e87bff32de345f0ff0f081b697d83ea374a0534375076bd
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
No
Uploaded via twine/6.1.0 CPython/3.9.22

Release history Release notifications | RSS feed

This release

2.4.28 This release

2 release files

2.4.27

2 release files

2.4.26

2 release files

2.4.25

2 release files

2.4.24

2 release files

2.4.23

2 release files

2.4.22

2 release files

2.4.21

2 release files

2.4.20

2 release files

2.4.19

2 release files

2.4.18

2 release files

2.4.17

2 release files

2.4.16

2 release files

2.4.15

2 release files

2.4.9

2 release files

2.4.8

2 release files

2.4.7

2 release files

2.4.6

2 release files

2.4.5

2 release files

2.4.4

2 release files

2.4.3

2 release files

2.4.2

2 release files

2.4.1

2 release files

2.4.0

2 release files

2.3.13

2 release files

2.3.12

2 release files

2.3.11

2 release files

2.3.10

2 release files

2.3.9

2 release files

2.3.8

2 release files

2.3.7

2 release files

2.3.6

2 release files

2.3.5

2 release files

2.3.4

2 release files

2.3.3

2 release files

2.3.2

2 release files

2.3.1

2 release files

2.3.0

2 release files

2.2.9

2 release files

2.2.8

2 release files

2.2.7

2 release files

2.2.6

2 release files

2.2.5

2 release files

2.2.4

2 release files

2.2.3

2 release files

2.2.2

2 release files

2.2.1

2 release files

2.2.0

2 release files

2.1.19

2 release files

2.1.18

2 release files

2.1.17

2 release files

2.1.16

2 release files

2.1.15

2 release files

2.1.14

2 release files

2.1.13

2 release files

2.1.12

2 release files

2.1.11

2 release files

2.1.10

2 release files

2.1.9

2 release files

2.1.8

2 release files

2.1.7

2 release files

2.1.6

2 release files

2.1.5

2 release files

2.1.4

2 release files

2.1.3

2 release files

2.1.2

2 release files

2.1.1

2 release files

2.1.0

2 release files

2.0.18

2 release files

2.0.17

2 release files

2.0.16

2 release files

2.0.15

2 release files

2.0.14

2 release files

2.0.13

2 release files

2.0.12

2 release files

2.0.11

2 release files

2.0.10

2 release files

2.0.9

2 release files

2.0.8

2 release files

2.0.7

2 release files

2.0.6

2 release files

2.0.5

2 release files

2.0.4

2 release files

2.0.3

2 release files

2.0.2

2 release files

2.0.1

2 release files

2.0.0

2 release files

1.9.9

2 release files

1.9.8

2 release files

1.9.7

2 release files

1.9.6

2 release files

1.9.5

2 release files

1.9.4

2 release files

1.9.3

2 release files

1.9.2

2 release files

1.9.1

2 release files

1.9.0

2 release files

1.8.4

2 release files

1.8.3

2 release files

1.8.2

2 release files

1.8.1

2 release files

1.8.0

2 release files

1.7.19

2 release files

1.7.18

2 release files

1.7.16

2 release files

1.7.15

2 release files

1.7.14

2 release files

1.7.13

2 release files

1.7.12

2 release files

1.7.11

2 release files

1.7.10

2 release files

1.7.9

2 release files

1.7.8

2 release files

1.7.6

2 release files

1.7.5

2 release files

1.7.4

2 release files

1.7.3

2 release files

1.7.2

2 release files

1.7.1

2 release files

1.7.0

2 release files

1.6.17

2 release files

1.6.16

2 release files

1.6.15

2 release files

1.6.14

2 release files

1.6.13

2 release files

1.6.9

2 release files

1.6.8

2 release files

1.6.7

2 release files

1.6.6

2 release files

1.6.5

2 release files

1.6.4

2 release files

1.6.3

2 release files

1.6.2

2 release files

1.6.1

2 release files

1.6.0

2 release files

1.5.9

2 release files

1.5.8

2 release files

1.5.7

2 release files

1.5.6

2 release files

1.5.5

2 release files

1.5.4

2 release files

1.5.3

2 release files

1.5.2

2 release files

1.5.1

2 release files

1.5.0

2 release files

1.4.21

2 release files

1.4.20

2 release files

1.4.19

2 release files

1.4.18

2 release files

1.4.17

2 release files

1.4.16

2 release files

1.4.15

2 release files

1.4.14

2 release files

1.4.13

2 release files

1.4.12

2 release files

1.4.11

2 release files

1.4.10

2 release files

1.4.9

2 release files

1.4.8

2 release files

1.4.7

2 release files

1.4.6

2 release files

1.4.5

2 release files

1.4.4

2 release files

1.4.3

2 release files

1.4.2

2 release files

1.4.1

2 release files

1.4.0

2 release files

1.3.8

2 release files

1.3.7

2 release files

1.3.6

2 release files

1.3.5

2 release files

1.3.4

2 release files

1.3.3

2 release files

1.3.2

2 release files

1.3.1

2 release files

1.3.0

2 release files

1.2.10

2 release files

1.2.9

2 release files

1.2.8

2 release files

1.2.7

2 release files

1.2.6

2 release files

1.2.5

2 release files

1.2.4

2 release files

1.2.3

2 release files

1.2.2

2 release files

1.2.1

2 release files

1.2.0

2 release files

1.1.19

2 release files

1.1.18

2 release files

1.1.17

2 release files

1.1.16

2 release files

1.1.15

2 release files

1.1.14

2 release files

1.1.13

2 release files

1.1.12

2 release files

1.1.11

2 release files

1.1.10

2 release files

1.1.9

2 release files

1.1.8

2 release files

1.1.7

2 release files

1.1.6

2 release files

1.1.5

2 release files

1.1.4

2 release files

1.1.3

2 release files

1.1.2

2 release files

1.1.1

2 release files

1.1.0

2 release files

1.0.0

2 release files

0.1.1

2 release files

0.1.0

2 release files

0.0.9

2 release files

0.0.8

2 release files

0.0.7

2 release files

0.0.6

2 release files

0.0.5

2 release files

0.0.4

2 release files

0.0.3

2 release files

Anthropic, PBC Visionary sponsor Bloomberg Visionary sponsor Hudson River Trading Visionary sponsor Meta Visionary sponsor NVIDIA Visionary sponsor Microsoft Sustainability sponsor Depot Continuous Integration AWS Cloud computing and Security Sponsor Datadog Monitoring Fastly CDN Google Download Analytics Sentry Error logging StatusPage Status page